cuecontext

package
v0.15.1 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Nov 21, 2025 License: Apache-2.0 Imports: 8 Imported by: 345

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func New

func New(options ...Option) *cue.Context

New creates a new *cue.Context.

The environment variables CUE_EXPERIMENT and CUE_DEBUG are followed to configure the evaluator, just like the cue tool documents via cue help environment. You can override these settings via options like EvaluatorVersion and CUE_DEBUG.

Types

type EvalVersion added in v0.9.0

type EvalVersion = internal.EvaluatorVersion
const (
	// EvalDefault is the default version of the evaluator, which is selected based on
	// the CUE_EXPERIMENT environment variable described in [cue help environment].
	//
	// [cue help environment]: https://cuelang.org/docs/reference/command/cue-help-environment/
	EvalDefault EvalVersion = internal.DefaultVersion

	// EvalDefault is the latest stable version of the evaluator, currently [EvalV3].
	EvalStable EvalVersion = internal.StableVersion

	// EvalExperiment refers to the latest in-development version of the evaluator,
	// currently [EvalV3]. Note that this version may change without notice.
	EvalExperiment EvalVersion = internal.DevVersion

	// EvalV3 is the current version of the evaluator. It was introduced in 2024
	// and brought a new disjunction algorithm, a new closedness algorithm, a
	// new core scheduler, and adds performance enhancements like structure sharing.
	EvalV3 EvalVersion = internal.EvalV3
)

type ExternInterpreter added in v0.6.0

type ExternInterpreter = runtime.Interpreter

An ExternInterpreter creates a compiler that can produce implementations of functions written in a language other than CUE. It is currently for internal use only.

type Option

type Option struct {
	// contains filtered or unexported fields
}

Option controls a build context.

func CUE_DEBUG added in v0.9.0

func CUE_DEBUG(s string) Option

CUE_DEBUG takes a string with the same contents as CUE_DEBUG and configures the context with the relevant debug options. It panics for unknown or malformed options.

func EvaluatorVersion added in v0.9.0

func EvaluatorVersion(v EvalVersion) Option

EvaluatorVersion indicates which version of the evaluator to use. Currently only experimental versions can be selected as an alternative.

func Interpreter added in v0.6.0

func Interpreter(i ExternInterpreter) Option

Interpreter associates an interpreter for external code with this context.

Jump to